/********************************************************************************
 Author : Sijo (cic.asia@analog.com)

 Date : 11-26-2013

 File name : AD5764.c

 Description : Function definition for AD5764 functions

 Hardware plateform : ADuCM360 	
********************************************************************************/

/*
ADuCM360 interface with AD5764

SYNC				- P1.0 
SCLK				- P1.1 
SDIN				- P1.2
SDOUT				- P1.3
LDAC				- P1.4
CLR					- P1.5 
BIN/2sCOMP	- P1.6
RSTIN				- P1.7

*/

#include <ADuCM360.h>
#include "ADuCM360Driver.h"
#include "AD5764.h"

extern unsigned char InputBuf[];
extern unsigned char OutputBuf[];

/**
	@brief unsigned char Sdout()
	- Configure P1.3 to Sdout returning the port value.
	return  SerialOut
	SerialOut -It will give bitwise Data out
**/
unsigned char Sdout()
{
	unsigned int SerialOut=0;
	SerialOut = ADuCM360GpioInput(SDOUT);
	return(SerialOut);
}

//Configure LDAC to Set
void AD5764SetLdac()
{
	ADuCM360GpioOutput(LDAC, PIN_HIGH);
}

//Configure LDAC to clear
void AD5764ClrLdac()
{
	ADuCM360GpioOutput(LDAC, PIN_LOW);
}

//Configure CLR to LOW
void AD5764Clr()
{
	ADuCM360GpioOutput(CLR, PIN_LOW);
}

//Configure CLR to HIGH
void AD5764Set()
{
	ADuCM360GpioOutput(LDAC, PIN_HIGH);
}

//Configure input coding format as Binary code
void AD5764BinCode()
{
	ADuCM360GpioOutput(BIN_COMP, PIN_HIGH);
}

//Configure input coding format as 2's Complement
void AD5764CompCode()
{
	ADuCM360GpioOutput(BIN_COMP, PIN_LOW);
}


//Configure RSTIN to HIGH
void AD5764SetRstin()
{
	ADuCM360GpioOutput(RSTIN, PIN_HIGH);
}

//configure RSTIN to LOW
void AD5764ClrRstin()
{
	ADuCM360GpioOutput(RSTIN, PIN_LOW);
}


/**
	@brief void AD5764Initialize(void)
	- Initialize AD5764 with default setting and values.
	@ no return  value
**/
void AD5764Initialize(void)
{
	AD5764BinCode(); //Setting input format as Binary
	AD5764ClrLdac();	//Setting LDAC pin to low, Output data will be updated in rising edge of SYNC
	
	SPIinitialize();
	
	//Resetting the DAC ouput Register
	AD5764ClrRstin();
	Delay(10);
	AD5764SetRstin();
	
	//Clearing the data register to 0x0000(falling edge of CLR)
	AD5764Clr();
	Delay(100);
	AD5764Set();
	Delay(10);
	AD5764Clr();
	
	//Write default values to all Registers

	WriteCoarseGainRegister(DAC_D,RANGE_10V);
	
	WriteFineGainRegister(DAC_ALL, 0x000000);
	
	WriteOffsetRegister(DAC_ALL, 0x000000);

}

/**
	@brief unsigned int WriteRegister(unsigned long int WriteData)
	- Write the data to the Register
		@param WriteData :{0x000000 - 0xFFFFFF}
		@ return 1 on successful write
		- return 0 on write failure
**/
void WriteRegister(unsigned long int WriteData)
{
	unsigned char Byte;
	
	for(Byte=0;Byte < REGISTER_SIZE;Byte++)
	{
		*(OutputBuf + Byte) = (WriteData>>(((REGISTER_SIZE-1)-Byte)*8)); //Writing the data to be written to 8 bit Output Buffer Array
		*(InputBuf + Byte) = 0x00;
	}
	SpiReadWrite( InputBuf, OutputBuf, REGISTER_SIZE); //Writes data to the specified register

}

/**
	@brief unsigned long int ReadRegister(unsigned long int RegisterSelect,unsigned long int DacSelect)
	- Reads the values of particular register of each channel.
		@param RegisterSelect :{REG_FUNCTION,REG_DATA,REG_COARSE_GAIN,REG_FINE_GAIN,REG_OFFSET}
		- REG_FUNCTION selects the Function Register
		- REG_DATA selects the Data Register
		- REG_COARSE_GAIN selects the Coarse Gain Register
		- REG_FINE_GAIN selects the Fine Gain Register
		- REG_OFFSET selects the Offset Register
		@param DacSelect :{DAC_A,DAC_B,DAC_C,DAC_D,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_C selects DAC C
		- DAC_D selects DAC D
		- DAC_ALL selects all DAC A,B,C,D
		@ return ReceivedData
		- ReceivedData - Read Data from the particular register from the particular Channel
**/
unsigned long int ReadRegister(unsigned long int RegisterSelect,unsigned long int DacSelect)
{
	
	unsigned long int ReadControl = 0x000000;
	unsigned int Byte;
	int RegisterData = 0x000000;
		
	RegisterData = WRITE + REG_FUNCTION + DAC_B + SDO_ENABLE; //ReadBack mode enabled
	WriteRegister(RegisterData);
	
	ReadControl = READ + RegisterSelect + DacSelect; // Converting the data WriteData into Read Data control
	for(Byte=0;Byte < REGISTER_SIZE;Byte++)
	{
		OutputBuf[Byte] = (ReadControl>>(((REGISTER_SIZE-1)- Byte)*8));//Register read control written to the 8 bit Output Buffer
		InputBuf[Byte] = 0x00; //Flushing the Input Buffer
		
	}
	
	SpiReadWrite( InputBuf, OutputBuf, REGISTER_SIZE); //ReadControl written and InputBuf neglected
	
	*OutputBuf = 0x00; //Nop condition for Reading
	
	SpiReadWrite( InputBuf, OutputBuf, REGISTER_SIZE); // Valid Received data will be available at InputBuf
	
	return *InputBuf;
}

/**
	@brief void WriteCoarseGainRegister(unsigned long int DacSelect, unsigned char Range)
	- Writes the values to 2 bit Coarse Gain Register, adjust the output range of each channel.
	@param DacSelect :{DAC_A,DAC_B,DAC_C,DAC_D,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_C selects DAC C
		- DAC_D selects DAC D
		- DAC_ALL selects all DAC A,B,C,D
	@param RANGE :{DAC_RANGE_10V/RANGE_10_2564V,RANGE_10_5263V} in 2's complement form
		- DAC_RANGE_10V selects the output range 10 V
		- RANGE_10_2564V selects the output range 10.2564 V
		- RANGE_10_5263V selects the output range 10.5263 V
			@ no return value
**/
void WriteCoarseGainRegister(unsigned long int DacSelect, unsigned char Range)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_COARSE_GAIN + DacSelect + Range; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}

/**
	@brief void WriteFineGainRegister(unsigned long int DacSelect, unsigned int FineGain)
	- Writes the values to 6 bit Fine Gain Register, adjust the gain of each channel by -32LSB's to +31LSB's in increment of 1 LSB.
	@param DacSelect :{DAC_A,DAC_B,DAC_C,DAC_D,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_C selects DAC C
		- DAC_D selects DAC D
		- DAC_ALL selects all DAC A,B,C,D
	@param Offset :{0x20-0x1F} in 2's complement form
			@ no return value
**/
void WriteFineGainRegister(unsigned long int DacSelect, unsigned int FineGain)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_FINE_GAIN + DacSelect + FineGain; ////Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}

/**
	@brief voidWriteOffsetRegister(unsigned long int DacSelect, unsigned int Offset)
	- Writes the values to 8 bit Offset Register, adjust the offsets of each channel by -16LSB's to +15.875LSB's in increment of 1/8 LSB.
	@param DacSelect :{DAC_A,DAC_B,DAC_C,DAC_D,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_C selects DAC C
		- DAC_D selects DAC D
		- DAC_ALL selects all DAC A,B,C,D
	@param Offset :{0x7F-0x80} in 2's complement form
			@ no return value
**/
void WriteOffsetRegister(unsigned long int DacSelect, unsigned int Offset)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_OFFSET + DacSelect + Offset; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}

//Function for writing Data Register
/**
	@brief void WriteDataRegister(unsigned long int DacSelect, unsigned int Data)
	- Writes the values to Data Register to specific DAC
	@param DacSelect :{DAC_A,DAC_B,DAC_C,DAC_D,DAC_ALL}
		- DAC_A selects DAC A
		- DAC_B selects DAC B
		- DAC_C selects DAC C
		- DAC_D selects DAC D
		- DAC_ALL selects all DAC A,B,C,D
	@param Data :{0x0000-0xFFFF}
			@ no return value
**/
void WriteDataRegister(unsigned long int DacSelect, unsigned int Data)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_DATA + DacSelect + Data; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}

/**
	@brief void ConfigureD0D1(unsigned long int D0direction, unsigned long int D1direction)
	- Configure the Digital I/O as output or input.
	@param D0direction :{D0_INPUT,D0_OUTPUT}
		- D0_INPUT for D0 as input
		- D0_OUTPUT for D0 as output
	@param D1direction :{D1_INPUT,D1_OUTPUT}
		- D1_INPUT for D1 as input
		- D1_OUTPUT for D1 as output
	@ no return value
**/
void ConfigureD0D1(unsigned long int D0direction, unsigned long int D1direction)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_FUNCTION + DAC_B + D1direction + D0direction; //Merging the data recieved into a single register
	WriteRegister(RegisterData);

}

/**
	@brief void OutD0D1(unsigned long int D0Value, unsigned long int D1Value)
	- Sets the Digital I/O to High or LOW when D0 and D1 are configured as output.
	@param D0Value :{D0_HIGH, D0_LOW}
		- D0_HIGH for D0 logic high
		- D0_LOW for D0 logic low
	@param D1Vlaue :{D1_HIGH,D1_LOW}
		- D1_HIGH for D1 logic high
		- D1_LOW for D1 logic low
	@ no return  value
**/
void OutD0D1(unsigned long int D0Value, unsigned long int D1Value)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_FUNCTION + DAC_B + D1_OUTPUT + D1Value + D0_OUTPUT + D0Value; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
}

/**
	@brief ClearData(void)
	-Resets the DAC output to 0V in 2's compliment mode and negative full scale in binary mode through software instead of CLR pin.
	@ no return  value
**/
void ClearData(void)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_FUNCTION + DAC_C; //Merging the data recieved into a single register
	WriteRegister(RegisterData);

}

/**
	@brief LoadData(void)
	-Updates the data to DAC Register through software instead of LDAC and gives corresponding analog voltage.

	@ no return  value
**/
void LoadData(void)
{
	int RegisterData = 0x000000;
	
	RegisterData = WRITE + REG_FUNCTION+ LOAD; //Merging the data recieved into a single register
	WriteRegister(RegisterData);
	
}
